@@ -1,4 +1,4 @@
-//===-- x86AssemblyInspectionEngine.cpp -------------------------*- C++ -*-===//
+//===-- x86AssemblyInspectionEngine.cpp -----------------------------------===//
 //
 // Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
 // See https://llvm.org/LICENSE.txt for license information.
@@ -24,11 +24,12 @@ x86AssemblyInspectionEngine::x86AssemblyInspectionEngine(const ArchSpec &arch)
     : m_cur_insn(nullptr), m_machine_ip_regnum(LLDB_INVALID_REGNUM),
       m_machine_sp_regnum(LLDB_INVALID_REGNUM),
       m_machine_fp_regnum(LLDB_INVALID_REGNUM),
+      m_machine_alt_fp_regnum(LLDB_INVALID_REGNUM),
       m_lldb_ip_regnum(LLDB_INVALID_REGNUM),
       m_lldb_sp_regnum(LLDB_INVALID_REGNUM),
       m_lldb_fp_regnum(LLDB_INVALID_REGNUM),
-
-      m_reg_map(), m_arch(arch), m_cpu(k_cpu_unspecified), m_wordsize(-1),
+      m_lldb_alt_fp_regnum(LLDB_INVALID_REGNUM), m_reg_map(), m_arch(arch),
+      m_cpu(k_cpu_unspecified), m_wordsize(-1),
       m_register_map_initialized(false), m_disasm_context() {
   m_disasm_context =
       ::LLVMCreateDisasm(arch.GetTriple().getTriple().c_str(), nullptr,
@@ -681,10 +682,6 @@ bool x86AssemblyInspectionEngine::jmp_to_reg_p() {
   // The second byte is a ModR/M /4 byte, strip off the registers
   uint8_t second_byte_sans_reg = *(m_cur_insn + 1) & ~7;
 
-  // Don't handle 0x24 disp32, because the target address is
-  // knowable statically - pc_rel_branch_or_jump_p() will
-  // return the target address.
-
   // [reg]
   if (second_byte_sans_reg == 0x20)
     return true;
@@ -701,17 +698,6 @@ bool x86AssemblyInspectionEngine::jmp_to_reg_p() {
   if (second_byte_sans_reg == 0xe0)
     return true;
 
-  // disp32
-  // jumps to an address stored in memory, the value can't be cached
-  // in an unwind plan.
-  if (second_byte_sans_reg == 0x24)
-    return true;
-
-  // use SIB byte
-  // ff 24 fe  jmpq   *(%rsi,%rdi,8)
-  if (second_byte_sans_reg == 0x24)
-    return true;
-
   return false;
 }
 
@@ -923,6 +909,9 @@ bool x86AssemblyInspectionEngine::GetNonCallSiteUnwindPlanFromAssembly(
   if (!m_register_map_initialized)
     return false;
 
+  if (m_disasm_context == nullptr)
+    return false;
+
   addr_t current_func_text_offset = 0;
   int current_sp_bytes_offset_from_fa = 0;
   bool is_aligned = false;
@@ -965,9 +954,9 @@ bool x86AssemblyInspectionEngine::GetNonCallSiteUnwindPlanFromAssembly(
   // path jumps over the mid-function epilogue
 
   UnwindPlan::RowSP prologue_completed_row; // copy of prologue row of CFI
-  int prologue_completed_sp_bytes_offset_from_cfa; // The sp value before the
+  int prologue_completed_sp_bytes_offset_from_cfa = 0; // The sp value before the
                                                    // epilogue started executed
-  bool prologue_completed_is_aligned;
+  bool prologue_completed_is_aligned = false;
   std::vector<bool> prologue_completed_saved_registers;
 
   while (current_func_text_offset < size) {
@@ -1584,6 +1573,9 @@ bool x86AssemblyInspectionEngine::FindFirstNonPrologueInstruction(
   if (!m_register_map_initialized)
     return false;
 
+  if (m_disasm_context == nullptr)
+    return false;
+
   while (offset < size) {
     int regno;
     int insn_len;
